/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/

/*
	Based on the Bemused protocol
	http://www.compsoc.man.ac.uk/~ashley/bemused/BemusedProtocol.html
	Client available from the same location.
*/

#ifndef __BLUETOOTH_CONTROL_H__
#define __BLUETOOTH_CONTROL_H__

#include <vector>

#include "connection.h"
#include "macros.h"
#include "control.h"

extern void g_debug(pfc::string8 str);



#define EMULATED_VERSION_MAJOR	1
#define EMULATED_VERSION_MINOR	73




/***************************
 * Directory Tree structure
 ***************************/
// upper nibble
#define DIRLST_ROOT		(0x0<<4)
#define DIRLST_CHILD	(0x1<<4)
#define DIRLST_OCHILD	(0x2<<4)
#define DIRLST_SIBLING	(0x3<<4)
#define DIRLST_LSIBLING	(0x4<<4)
#define DIRLST_TERM		(0xF<<4)
// lower nibble
#define DIRLST_DRIVE	0x0
#define DIRLST_DIR		0x1
#define DIRLST_FILE		0x2
#define DIRLST_UXDIR	0x3

class DirTreeNode
{
public:
	unsigned char				type;	// upper and lower nibbles
	DirTreeNode*				first_child;
	std::vector<DirTreeNode*>	siblings;
	pfc::string8						name;
	DirTreeNode(pfc::string8 n, unsigned char t) {
		name = n;
		type = t;
		first_child = NULL;
	}
};



class bluetooth_control
{
public:
	bluetooth_control(connection *conn, control *play_control);
	~bluetooth_control();

	void send_command(unsigned long cmd);
	bool set_active(bool activate);
	void set_musicdir(pfc::string8 musicdir);
	void set_random(bool random);

private:
	connection *m_conn;
	control	*m_play_control;
	bool	m_activated;
	metadb	*m_mdb;
	HANDLE	m_dirlist_thread;

	pfc::string8 m_musicdir;
	bool m_random;



	void debug(pfc::string8 str);

	static unsigned long WINAPI thread_send_entire_dirlist(LPVOID param);
	/**
	 * The Bemused protocol can be found at:
	 * http://www.compsoc.man.ac.uk/~ashley/bemused/BemusedProtocol.html
	 */
	void process_command(unsigned long _cmd);
	pfc::string8 read_filename();

	DirTreeNode* build_dirtree(pfc::string8 root, pfc::string8 prefix, unsigned char type, bool recursive);
	void delete_dirtree(DirTreeNode *root);
	void send_detailed_info();
	void append_dirlist(buffer &buf, DirTreeNode *root);
	void download_file(pfc::string8 fname);
	void send_fileinfo(pfc::string8 fname);
	void send_volume();
	void send_songinfo();
	void send_songinfo2();
	void send_entire_list();
	void send_partial_list(pfc::string8 dir);
	void send_playlist_length();
	void send_playlist();
	void send_version();
};



#endif /*__BLUETOOTH_CONTROL_H__*/