/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#ifndef __CONNECTION_H__
#define __CONNECTION_H__

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"

#include "macros.h"
#include "buffer.h"


/**
 * connection interface
 */
class connection
{
public:
	virtual bool connect()=0;
	virtual void disconnect()=0;
	virtual int send(buffer &buf) = 0;
	virtual int send(void *data, int length) = 0;
	virtual int read(void *dest, int length) = 0;
	virtual void set_parameter(pfc::string8 str) {};
};


/**
 * bluetooth connection class
 */
class bluetooth_connection : public connection
{
public:
	bluetooth_connection();
	bool connect();
	void disconnect();
	int send(buffer &buf);
	int send(void *data, int length);
	int read(void *dest, int length);
	void set_parameter(pfc::string8 str);
private:
	pfc::string8 m_port;
	HANDLE m_port_handle;
};

/**
 * socket connection class for testing purposes
 */
class socket_connection : public connection
{
public:
	socket_connection();
	bool connect();
	void disconnect();
	int send(buffer &buf);
	int send(void *data, int length);
	int read(void *dest, int length);
	void set_parameter(pfc::string8 str);
private:
	SOCKET m_socket;
};


#endif /*__CONNECTION_H__*/