/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#ifndef __CONTROL_H__
#define __CONTROL_H__

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"

#include <vector>



class control
{
public:
	virtual void fade_stop()=0;					// fade and stop
	virtual void ffwd(long secs)=0;				// fast forward
	virtual void rwnd(long secs)=0;				// rewind
	virtual void next(bool random)=0;			// advance in playlist
	virtual void prev()=0;						// previous in playlist
	virtual void pause()=0;						// pause
	virtual void stop()=0;						// stop immediately
	virtual void play(pfc::string8 fname)=0;	// play file
	virtual void play_current()=0;				// play current file in playlist
	virtual long get_playlist_len()=0;			// returns the length of the playlist
	virtual std::vector<pfc::string8> *get_playlist()=0;// returns a vector with the playlist
	virtual void add_to_playlist(pfc::string8 fname)=0;// add a file to the playlist
	virtual void select_at(int idx)=0;			// select the song at idx in playlist
	virtual void clear_playlist()=0;			// clear the playlist
	virtual void set_repeat(bool repeat)=0;		// set repeat
	virtual void set_shuffle(bool shuffle)=0;	// set shuffle
	virtual void seek(long pos)=0;				// seek to position
	virtual void set_stop_at_eof_track(bool stop)=0;// guess
	virtual void set_volume(unsigned char vol)=0;// set volume
	virtual unsigned char get_volume()=0;		// return volume
	virtual long get_bitrate()=0;				// return bitrate of current song
	virtual long get_samplerate()=0;			// return samplerate of current song
	virtual long get_channels()=0;				// return number of channels of current song
	virtual const char* get_file_path()=0;		// return the path of the current song
	virtual long get_playback_time()=0;			// return played seconds
	virtual long get_playlength()=0;			// return playlength
	virtual bool get_shuffle()=0;				// return shuffle state
	virtual bool get_repeat()=0;				// return repeat state
	virtual void get_title(pfc::string8 &str)=0;// return the title of the current song
	virtual bool is_paused()=0;					// return if paused
	virtual bool is_playing()=0;				// return if playing
	virtual long get_cur_idx()=0;				// return current playlist index
};






#endif /*__CONTROL_H__*/