/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#include "connection.h"


socket_connection::socket_connection()
{
	m_socket = -1;
}


bool socket_connection::connect()
{
	WSADATA wsaData;
	int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
	if (iResult != NO_ERROR)
		console::info("Error at WSAStartup()\n");

	pfc::string8 tmpstr ("Connecting on port 9999");
	console::info(tmpstr);

	SOCKET s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s < 0) {
		WSACleanup();
		return false;
	}
	
	struct sockaddr_in saddr;
	memset(&saddr, 0, sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(9999);
	saddr.sin_addr.s_addr = INADDR_ANY;

	if (bind(s, (const sockaddr *)&saddr, sizeof(saddr)) == SOCKET_ERROR) {
		console::info("bind() failed.\n");
		closesocket(s);
		return false;
	}

	listen(s, 5);

	while(true) {
		m_socket = SOCKET_ERROR;
		while (m_socket == SOCKET_ERROR) 
			m_socket = accept(s, NULL, NULL);
		break;
	};
	return true;
}


void socket_connection::disconnect()
{
	if(m_socket >= 0) {
		closesocket(m_socket);
		m_socket = -1;
	}
	WSACleanup();
}


int socket_connection::send(void *data, int length)
{
//	console::info((const char*)data);
	return ::send(m_socket, (const char*)data, length, 0);
}


int socket_connection::read(void *dest, int length)
{
	int len = recv(m_socket, (char*)dest, length, 0);
//	console::info((const char *)dest);
	return len;
}


int socket_connection::send(buffer &buf)
{
//	console::info((const char*)buf.get_ptr());
	return ::send(m_socket, (const char*) buf.get_ptr(), buf.length(), 0);
}


void socket_connection::set_parameter(pfc::string8 str)
{
}
